% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ru_sampling.R
\name{ru}
\alias{ru}
\title{Generalized ratio-of-uniforms sampling}
\usage{
ru(
  logf,
  ...,
  n = 1,
  d = 1,
  init = NULL,
  mode = NULL,
  trans = c("none", "BC", "user"),
  phi_to_theta = NULL,
  log_j = NULL,
  user_args = list(),
  lambda = rep(1L, d),
  lambda_tol = 1e-06,
  gm = NULL,
  rotate = ifelse(d == 1, FALSE, TRUE),
  lower = rep(-Inf, d),
  upper = rep(Inf, d),
  r = 1/2,
  ep = 0L,
  a_algor = if (d == 1) "nlminb" else "optim",
  b_algor = c("nlminb", "optim"),
  a_method = c("Nelder-Mead", "BFGS", "CG", "L-BFGS-B", "SANN", "Brent"),
  b_method = c("Nelder-Mead", "BFGS", "CG", "L-BFGS-B", "SANN", "Brent"),
  a_control = list(),
  b_control = list(),
  var_names = NULL,
  shoof = 0.2
)
}
\arguments{
\item{logf}{A function returning the log of the target density \eqn{f}
evaluated at its first argument.
This function should return \code{-Inf} when the density is zero.
It is better to use \code{logf = } explicitly, for example,
\code{ru(logf = dnorm, log = TRUE, init = 0.1)},
to avoid argument matching problems.  In contrast,
\code{ru(dnorm, log = TRUE, init = 0.1)}
will throw an error because partial matching results in
\code{logf} being matched to \code{log = TRUE}.}

\item{...}{Further arguments to be passed to \code{logf} and related
functions.}

\item{n}{A non-negative integer scalar.  The number of simulated values
required. If \code{n = 0} then no simulation is performed but the
component \code{box} in the returned object gives the ratio-of-uniforms
bounding box that would have been used.}

\item{d}{A positive integer scalar. The dimension of \eqn{f}.}

\item{init}{A numeric vector of length \code{d}. Initial estimate of the
mode of \code{logf}.
If \code{trans = "BC"} or \code{trans = "user"} this is \emph{after}
Box-Cox transformation or user-defined transformation, but \emph{before}
any rotation of axes.
If \code{init} is not supplied then \code{rep(1, d)} is used.
If \code{length(init) = 1} and \code{d > 1} then
\code{init <- rep(init, length.out = d)} is used.}

\item{mode}{A numeric vector of length \code{d}.  The mode of \code{logf}.
If \code{trans = "BC"} or \code{trans = "user"} this is \emph{after}
Box-Cox transformation or user-defined transformation, but \emph{before}
any rotation of axes.  Only supply \code{mode} if the mode is known: it
will not be checked.  If \code{mode} is supplied then \code{init} is
ignored.}

\item{trans}{A character scalar. \code{trans = "none"} for no
transformation, \code{trans = "BC"} for Box-Cox transformation,
\code{trans = "user"} for a user-defined transformation.
If \code{trans = "user"} then the transformation should be specified
using \code{phi_to_theta} and \code{log_j} and \code{user_args} may be
used to pass arguments to \code{phi_to_theta} and \code{log_j}.
See \strong{Details} and the \strong{Examples}.}

\item{phi_to_theta}{A function returning (the inverse) of the transformation
from \code{theta} (\eqn{\theta}) to \code{phi} (\eqn{\phi}) that may be
used to ensure positivity of \eqn{\phi} prior to Box-Cox transformation.
The argument is \code{phi} and the returned value is \code{theta}.
If \code{phi_to_theta} is undefined at the input value then the
function should return \code{NA}. See \strong{Details}.
If \code{lambda$phi_to_theta} (see argument \code{lambda} below) is
supplied then this is used instead of any function supplied via
\code{phi_to_theta}.}

\item{log_j}{A function returning the log of the Jacobian of the
transformation from \code{theta} (\eqn{\theta}) to \code{phi} (\eqn{\phi}),
i.e., based on derivatives of \eqn{\phi} with respect to \eqn{\theta}.
Takes \code{theta} as its argument.
If \code{lambda$log_j} (see argument \code{lambda} below) is
supplied then this is used instead of any function supplied via
\code{log_j}.}

\item{user_args}{A list of numeric components. If \code{trans = "user"}
then \code{user_args} is a list providing arguments to the user-supplied
functions \code{phi_to_theta} and \code{log_j}.}

\item{lambda}{Either
\itemize{
  \item A numeric vector.  Box-Cox transformation parameters, or
  \item A list with components
  \describe{
    \item{lambda}{A numeric vector.  Box-Cox parameters (required).}
    \item{gm}{A numeric vector.  Box-Cox scaling parameters (optional).
      If supplied this overrides any \code{gm} supplied by the individual
      \code{gm} argument described below.}
    \item{init_psi}{A numeric vector.  Initial estimate of mode \emph{after}
      Box-Cox transformation (optional).}
    \item{sd_psi}{A numeric vector.  Estimates of the marginal standard
      deviations of the Box-Cox transformed variables (optional).}
    \item{phi_to_theta}{As above (optional).}
    \item{log_j}{As above (optional).}
  }
  This list may be created using \link{find_lambda_one_d} (for \code{d} = 1)
  or \link{find_lambda} (for any \code{d}).
}}

\item{lambda_tol}{A numeric scalar.  Any values in lambda that are less
than \code{lambda_tol} in magnitude are set to zero.}

\item{gm}{A numeric vector. Box-Cox scaling parameters (optional). If
\code{lambda$gm} is supplied in input list \code{lambda} then
\code{lambda$gm} is used, not \code{gm}.}

\item{rotate}{A logical scalar. If TRUE (\code{d} > 1 only) use Choleski
rotation.  If d = 1 and \code{rotate = TRUE} then rotate will be set to
FALSE with a warning. See \strong{Details}.}

\item{lower, upper}{Numeric vectors.  Lower/upper bounds on the arguments of
the function \emph{after} any transformation from theta to phi implied by
the inverse of \code{phi_to_theta}. If \code{rotate = FALSE} these
are used in all of the optimisations used to construct the bounding box.
If \code{rotate = TRUE} then they are use only in the first optimisation
to maximise the target density.`
If \code{trans = "BC"} components of \code{lower} that are negative are
set to zero without warning and the bounds implied after the Box-Cox
transformation are calculated inside \code{ru}.}

\item{r}{A numeric scalar.  Parameter of generalized ratio-of-uniforms.}

\item{ep}{A numeric scalar.  Controls initial estimates for optimisations
to find the \eqn{b}-bounding box parameters.  The default (\code{ep} = 0)
corresponds to starting at the mode of \code{logf} small positive values
of \code{ep} move the constrained variable slightly away from the mode in
the correct direction.  If \code{ep} is negative its absolute value is
used, with no warning given.}

\item{a_algor, b_algor}{Character scalars.  Either \code{"nlminb"} or
\code{"optim"}.
Respective optimisation algorithms used to find \eqn{a(r)} and
(\ifelse{html}{\eqn{b}\out{<sub>i</sub>}\out{<sup>-</sup>}(r)}{\eqn{b_i^-(r)}},
\ifelse{html}{\eqn{b}\out{<sub>i</sub>}\out{<sup>+</sup>}(r)}{\eqn{b_i^+(r)}}).}

\item{a_method, b_method}{Character scalars.  Respective methods used by
\code{optim} to find \eqn{a(r)} and
(\ifelse{html}{\eqn{b}\out{<sub>i</sub>}\out{<sup>-</sup>}(r)}{\eqn{b_i^-(r)}},
\ifelse{html}{\eqn{b}\out{<sub>i</sub>}\out{<sup>+</sup>}(r)}{\eqn{b_i^+(r)}}).
Only used if \code{optim} is the chosen algorithm.  If \code{d} = 1 then
\code{a_method} and \code{b_method} are set to \code{"Brent"} without
warning.}

\item{a_control, b_control}{Lists of control arguments to \code{optim} or
\code{nlminb} to find \eqn{a(r)} and
(\ifelse{html}{\eqn{b}\out{<sub>i</sub>}\out{<sup>-</sup>}(r)}{\eqn{b_i^-(r)}},
\ifelse{html}{\eqn{b}\out{<sub>i</sub>}\out{<sup>+</sup>}(r)}{\eqn{b_i^+(r)}})
respectively.}

\item{var_names}{A character (or numeric) vector of length \code{d}.  Names
to give to the column(s) of the simulated values.}

\item{shoof}{A numeric scalar in [0, 1].  Sometimes a spurious
non-zero convergence indicator is returned from
\code{\link[stats]{optim}} or \code{\link[stats]{nlminb}}).
In this event we try to check that a minimum has indeed been found using
different algorithm.  \code{shoof} controls the starting value provided
to this algorithm.
If \code{shoof = 0} then we start from the current solution.
If \code{shoof = 1} then we start from the initial estimate provided
to the previous minimisation.  Otherwise, \code{shoof} interpolates
between these two extremes, with a value close to zero giving a starting
value that is close to the current solution.
The exception to this is when the initial and current solutions are equal.
Then we start from the current solution multiplied by \code{1 - shoof}.}
}
\value{
An object of class \code{"ru"} is a list containing the following
  components:
    \item{sim_vals}{An \code{n} by \code{d} matrix of simulated values.}
    \item{box}{A (2 * \code{d} + 1) by \code{d} + 2 matrix of
      ratio-of-uniforms bounding box information, with row names indicating
      the box parameter.  The columns contain
      \describe{
        \item{column 1}{values of box parameters.}
        \item{columns 2 to (2+\code{d}-1)}{values of variables at which
         these box parameters are obtained.}
        \item{column 2+\code{d}}{convergence indicators.}
      }
      Scaling of f within \code{ru} and relocation of the
      mode to the origin means that the first row of \code{box} will always
      be \code{c(1, rep(0, d))}.
    }
    \item{pa}{A numeric scalar.  An estimate of the probability of
      acceptance.}
    \item{r}{The value of \code{r}.}
    \item{d}{The value of \code{d}.}
    \item{logf}{A function. \code{logf} supplied by the user, but
      with f scaled by the maximum of the target density used in the
      ratio-of-uniforms method (i.e. \code{logf_rho}), to avoid numerical
      problems in contouring f in \code{\link{plot.ru}} when
      \code{d = 2}.}
    \item{logf_rho}{A function. The target function actually used in the
      ratio-of-uniforms algorithm.}
    \item{sim_vals_rho}{An \code{n} by \code{d} matrix of values simulated
      from the function used in the ratio-of-uniforms algorithm.}
    \item{logf_args}{A list of further arguments to \code{logf}.}
    \item{f_mode}{The estimated mode of the target density f, after any
      Box-Cox transformation and/or user supplied transformation, but before
      mode relocation.}
    \item{trans_fn}{An R function that performs the inverse transformation
      from the transformed variable \eqn{\rho}, on which the generalised
      ratio-of-uniforms method is performed, back to the original variable
      \eqn{\theta}. \strong{Note}: \code{trans_fn} is \strong{not}
      vectorised with respect to \eqn{\rho}.}
}
\description{
Uses the generalized ratio-of-uniforms method to simulate from a
distribution with log-density \eqn{\log f}{log f} (up to an additive
constant). The density \eqn{f} must be bounded, perhaps after a
transformation of variable.
}
\details{
For information about the generalised ratio-of-uniforms method and
  transformations see the
  \href{https://paulnorthrop.github.io/rust/articles/rust-a-vignette.html}{
  Introducing rust} vignette.  This can also be accessed using
  \code{vignette("rust-a-vignette", package = "rust")}.

  If \code{trans = "none"} and \code{rotate = FALSE} then \code{ru}
  implements the (multivariate) generalized ratio of uniforms method
  described in Wakefield, Gelfand and Smith (1991) using a target
  density whose mode is relocated to the origin (`mode relocation') in the
  hope of increasing efficiency.

  If \code{trans = "BC"} then marginal Box-Cox transformations of each of
  the \code{d} variables is performed, with parameters supplied in
  \code{lambda}.  The function \code{phi_to_theta} may be used, if
  necessary, to ensure positivity of the variables prior to Box-Cox
  transformation.

  If \code{trans = "user"} then the function \code{phi_to_theta} enables
  the user to specify their own transformation.

  In all cases the mode of the target function is relocated to the origin
  \emph{after} any user-supplied transformation and/or Box-Cox
  transformation.

  If \code{d} is greater than one and \code{rotate = TRUE} then a rotation
  of the variable axes is performed \emph{after} mode relocation.  The
  rotation is based on the Choleski decomposition (see \link{chol}) of the
  estimated Hessian (computed using \code{\link[stats:optim]{optimHess}}
  of the negated
  log-density after any user-supplied transformation or Box-Cox
  transformation.  If any of the eigenvalues of the estimated Hessian are
  non-positive (which may indicate that the estimated mode of \code{logf}
  is close to a variable boundary) then \code{rotate} is set to \code{FALSE}
  with a warning.  A warning is also given if this happens when
  \code{d} = 1.

  The default value of the tuning parameter \code{r} is 1/2, which is
  likely to be close to optimal in many cases, particularly if
  \code{trans = "BC"}.
}
\examples{
# Normal density ===================

# One-dimensional standard normal ----------------
x <- ru(logf = function(x) -x ^ 2 / 2, d = 1, n = 1000, init = 0.1)

# Two-dimensional standard normal ----------------
x <- ru(logf = function(x) -(x[1]^2 + x[2]^2) / 2, d = 2, n = 1000,
        init = c(0, 0))

# Two-dimensional normal with positive association ----------------
rho <- 0.9
covmat <- matrix(c(1, rho, rho, 1), 2, 2)
log_dmvnorm <- function(x, mean = rep(0, d), sigma = diag(d)) {
  x <- matrix(x, ncol = length(x))
  d <- ncol(x)
  - 0.5 * (x - mean) \%*\% solve(sigma) \%*\% t(x - mean)
}

# No rotation.
x <- ru(logf = log_dmvnorm, sigma = covmat, d = 2, n = 1000, init = c(0, 0),
        rotate = FALSE)

# With rotation.
x <- ru(logf = log_dmvnorm, sigma = covmat, d = 2, n = 1000, init = c(0, 0))

# three-dimensional normal with positive association ----------------
covmat <- matrix(rho, 3, 3) + diag(1 - rho, 3)

# No rotation.  Slow !
x <- ru(logf = log_dmvnorm, sigma = covmat, d = 3, n = 1000,
        init = c(0, 0, 0), rotate = FALSE)

# With rotation.
x <- ru(logf = log_dmvnorm, sigma = covmat, d = 3, n = 1000,
        init = c(0, 0, 0))

# Log-normal density ===================

# Sampling on original scale ----------------
x <- ru(logf = dlnorm, log = TRUE, d = 1, n = 1000, lower = 0, init = 1)

# Box-Cox transform with lambda = 0 ----------------
lambda <- 0
x <- ru(logf = dlnorm, log = TRUE, d = 1, n = 1000, lower = 0, init = 0.1,
        trans = "BC", lambda = lambda)

# Equivalently, we could use trans = "user" and supply the (inverse) Box-Cox
# transformation and the log-Jacobian by hand
x <- ru(logf = dlnorm, log = TRUE, d = 1, n = 1000, init = 0.1,
        trans = "user", phi_to_theta = function(x) exp(x),
        log_j = function(x) -log(x))

# Gamma(alpha, 1) density ===================

# Note: the gamma density in unbounded when its shape parameter is < 1.
# Therefore, we can only use trans="none" if the shape parameter is >= 1.

# Sampling on original scale ----------------

alpha <- 10
x <- ru(logf = dgamma, shape = alpha, log = TRUE, d = 1, n = 1000,
        lower = 0, init = alpha)

alpha <- 1
x <- ru(logf = dgamma, shape = alpha, log = TRUE, d = 1, n = 1000,
        lower = 0, init = alpha)

# Box-Cox transform with lambda = 1/3 works well for shape >= 1. -----------

alpha <- 1
x <- ru(logf = dgamma, shape = alpha, log = TRUE, d = 1, n = 1000,
        trans = "BC", lambda = 1/3, init = alpha)
summary(x)

# Equivalently, we could use trans = "user" and supply the (inverse) Box-Cox
# transformation and the log-Jacobian by hand

# Note: when phi_to_theta is undefined at x this function returns NA
phi_to_theta  <- function(x, lambda) {
  ifelse(x * lambda + 1 > 0, (x * lambda + 1) ^ (1 / lambda), NA)
}
log_j <- function(x, lambda) (lambda - 1) * log(x)
lambda <- 1/3
x <- ru(logf = dgamma, shape = alpha, log = TRUE, d = 1, n = 1000,
        trans = "user", phi_to_theta = phi_to_theta, log_j = log_j,
        user_args = list(lambda = lambda), init = alpha)
summary(x)

\donttest{
# Generalized Pareto posterior distribution ===================

# Sample data from a GP(sigma, xi) distribution
gpd_data <- rgpd(m = 100, xi = -0.5, sigma = 1)
# Calculate summary statistics for use in the log-likelihood
ss <- gpd_sum_stats(gpd_data)
# Calculate an initial estimate
init <- c(mean(gpd_data), 0)

# Mode relocation only ----------------
n <- 1000
x1 <- ru(logf = gpd_logpost, ss = ss, d = 2, n = n, init = init,
         lower = c(0, -Inf), rotate = FALSE)
plot(x1, xlab = "sigma", ylab = "xi")
# Parameter constraint line xi > -sigma/max(data)
# [This may not appear if the sample is far from the constraint.]
abline(a = 0, b = -1 / ss$xm)
summary(x1)

# Rotation of axes plus mode relocation ----------------
x2 <- ru(logf = gpd_logpost, ss = ss, d = 2, n = n, init = init,
         lower = c(0, -Inf))
plot(x2, xlab = "sigma", ylab = "xi")
abline(a = 0, b = -1 / ss$xm)
summary(x2)

# Cauchy ========================

# The bounding box cannot be constructed if r < 1.  For r = 1 the
# bounding box parameters b1-(r) and b1+(r) are attained in the limits
# as x decreases/increases to infinity respectively.  This is fine in
# theory but using r > 1 avoids this problem and the largest probability
# of acceptance is obtained for r approximately equal to 1.26.

res <- ru(logf = dcauchy, log = TRUE, init = 0, r = 1.26, n = 1000)

# Half-Cauchy ===================

log_halfcauchy <- function(x) {
  return(ifelse(x < 0, -Inf, dcauchy(x, log = TRUE)))
}

# Like the Cauchy case the bounding box cannot be constructed if r < 1.
# We could use r > 1 but the mode is on the edge of the support of the
# density so as an alternative we use a log transformation.

x <- ru(logf = log_halfcauchy, init = 0, trans = "BC", lambda = 0, n = 1000)
x$pa
plot(x, ru_scale = TRUE)

# Example 4 from Wakefield et al. (1991) ===================

# Bivariate normal x bivariate student-t
log_norm_t <- function(x, mean = rep(0, d), sigma1 = diag(d), sigma2 = diag(d)) {
  x <- matrix(x, ncol = length(x))
  log_h1 <- -0.5 * (x - mean) \%*\% solve(sigma1) \%*\% t(x - mean)
  log_h2 <- -2 * log(1 + 0.5 * x \%*\% solve(sigma2) \%*\% t(x))
  return(log_h1 + log_h2)
}

rho <- 0.9
covmat <- matrix(c(1, rho, rho, 1), 2, 2)
y <- c(0, 0)

# Case in the top right corner of Table 3
x <- ru(logf = log_norm_t, mean = y, sigma1 = covmat, sigma2 = covmat,
  d = 2, n = 10000, init = y, rotate = FALSE)
x$pa

# Rotation increases the probability of acceptance
x <- ru(logf = log_norm_t, mean = y, sigma1 = covmat, sigma2 = covmat,
  d = 2, n = 10000, init = y, rotate = TRUE)
x$pa

# Normal x log-normal: different Box-Cox parameters ==================
norm_lognorm <- function(x, ...) {
  dnorm(x[1], ...) + dlnorm(x[2], ...)
}
x <- ru(logf = norm_lognorm, log = TRUE, n = 1000, d = 2, init = c(-1, 0),
        trans = "BC", lambda = c(1, 0))
plot(x)
plot(x, ru_scale = TRUE)
}
}
\references{
Wakefield, J. C., Gelfand, A. E. and Smith, A. F. M. (1991)
 Efficient generation of random variates via the ratio-of-uniforms method.
 \emph{Statistics and Computing} (1991), \strong{1}, 129-133.
 \doi{10.1007/BF01889987}.
}
\seealso{
\code{\link{ru_rcpp}} for a version of \code{\link{ru}} that uses
  the Rcpp package to improve efficiency.

\code{\link{summary.ru}} for summaries of the simulated values
  and properties of the ratio-of-uniforms algorithm.

\code{\link{plot.ru}} for a diagnostic plot.

\code{\link{find_lambda_one_d}} to produce (somewhat) automatically
  a list for the argument \code{lambda} of \code{ru} for the
  \code{d} = 1 case.

\code{\link{find_lambda}} to produce (somewhat) automatically
  a list for the argument \code{lambda} of \code{ru} for any value of
  \code{d}.

\code{\link[stats]{optim}} for choices of the arguments
  \code{a_method}, \code{b_method}, \code{a_control} and \code{b_control}.

\code{\link[stats]{nlminb}} for choices of the arguments
  \code{a_control} and \code{b_control}.

\code{\link[stats:optim]{optimHess}} for Hessian estimation.

\code{\link[base]{chol}} for the Choleski decomposition.
}
